@@ -58,7 +58,7 @@ RRModeCreate (xRRModeInfo   *modeInfo,
     if (!RRInit ())
 	return NULL;
 
-    mode = xalloc (sizeof (RRModeRec) + modeInfo->nameLength + 1);
+    mode = malloc(sizeof (RRModeRec) + modeInfo->nameLength + 1);
     if (!mode)
 	return NULL;
     mode->refcnt = 1;
@@ -69,13 +69,13 @@ RRModeCreate (xRRModeInfo   *modeInfo,
     mode->userScreen = userScreen;
 
     if (num_modes)
-	newModes = xrealloc (modes, (num_modes + 1) * sizeof (RRModePtr));
+	newModes = realloc(modes, (num_modes + 1) * sizeof (RRModePtr));
     else
-	newModes = xalloc (sizeof (RRModePtr));
+	newModes = malloc(sizeof (RRModePtr));
 
     if (!newModes)
     {
-	xfree (mode);
+	free(mode);
 	return NULL;
     }
 
@@ -164,7 +164,7 @@ RRModesForScreen (ScreenPtr pScreen, int *num_ret)
     RRModePtr	*screen_modes;
     int		num_screen_modes = 0;
 
-    screen_modes = xalloc ((num_modes ? num_modes : 1) * sizeof (RRModePtr));
+    screen_modes = malloc((num_modes ? num_modes : 1) * sizeof (RRModePtr));
     if (!screen_modes)
 	return NULL;
     
@@ -243,14 +243,14 @@ RRModeDestroy (RRModePtr mode)
 	    num_modes--;
 	    if (!num_modes)
 	    {
-		xfree (modes);
+		free(modes);
 		modes = NULL;
 	    }
 	    break;
 	}
     }
     
-    xfree (mode);
+    free(mode);
 }
 
 static int
@@ -268,6 +268,7 @@ RRModeInit (void)
     RRModeType = CreateNewResourceType (RRModeDestroyResource, "MODE");
     if (!RRModeType)
 	return FALSE;
+    SetResourceTypeErrorValue(RRModeType, RRErrorBase + BadRRMode);
     return TRUE;
 }
 
@@ -320,7 +321,7 @@ ProcRRCreateMode (ClientPtr client)
     WriteToClient(client, sizeof(xRRCreateModeReply), (char *)&rep);
     /* Drop out reference to this mode */
     RRModeDestroy (mode);
-    return client->noClientException;
+    return Success;
 }
 
 int
